@@ -320,7 +320,7 @@ public final class Binder {
             arg_o = createBox(tc, gcx, origArg, gotNative);
         }
         else {
-            /* We need to do an unboxing opeation. */
+            /* We need to do an unboxing operation. */
             SixModelObject decontValue = Ops.decont((SixModelObject)origArg, tc);
             StorageSpec spec = decontValue.st.REPR.get_storage_spec(tc, decontValue.st);
             switch (desiredNative) {
@@ -394,7 +394,7 @@ public final class Binder {
             if (!noNomTypeCheck) {
                 /* Is the nominal type generic and in need of instantiation? (This
                  * can happen in (::T, T) where we didn't learn about the type until
-                 * during the signature bind). */
+                 * during the signature bind.) */
                 nomType = param.get_attribute_boxed(tc, gcx.Parameter,
                     "$!nominal_type", HINT_nominal_type);
                 if ((paramFlags & SIG_ELEM_NOMINAL_GENERIC) != 0) {
@@ -833,7 +833,18 @@ public final class Binder {
                 return res;
             }
             else {
-                return param.get_attribute_boxed(tc, gcx.Parameter, "$!nominal_type", HINT_nominal_type);
+                param.get_attribute_native(tc, gcx.Parameter, "$!flags", HINT_flags);
+                int paramFlags = (int)tc.native_i;
+                switch (paramFlags & SIG_ELEM_NATIVE_VALUE) {
+                    case SIG_ELEM_NATIVE_INT_VALUE:
+                        return createBox(tc, gcx, (long)0, CallSiteDescriptor.ARG_INT);
+                    case SIG_ELEM_NATIVE_NUM_VALUE:
+                        return createBox(tc, gcx, (double)0.0, CallSiteDescriptor.ARG_NUM);
+                    case SIG_ELEM_NATIVE_STR_VALUE:
+                        return createBox(tc, gcx, null, CallSiteDescriptor.ARG_STR);
+                    default:
+                        return param.get_attribute_boxed(tc, gcx.Parameter, "$!nominal_type", HINT_nominal_type);
+                }
             }
         }
     }
@@ -984,7 +995,7 @@ public final class Binder {
 
                 /* Otherwise, a positional. */
                 else {
-                    /* Do we have a value?. */
+                    /* Do we have a value? */
                     if (curPosArg < numPosArgs) {
                         /* Easy - just bind that. */
                         bindFail = bindOneParam(tc, gcx, cf, param, args[curPosArg],
